# -----------------------------------------------------------------------------
#   $RCSfile: Shopping.mak,v $
#  
#   See Copyright for the status of this software.
#  
#   The OpenSOAP Project
#   http://opensoap.jp/
# -----------------------------------------------------------------------------

PROGRAMS = ShoppingClient ShoppingService ShoppingService.cgi

OPENSOAP_PREFIX=/usr/local
OPENSOAP_INCLUDE=$(OPENSOAP_PREFIX)/include
OPENSOAP_LIB=$(OPENSOAP_PREFIX)/lib

CFLAGS=-I$(OPENSOAP_INCLUDE)
# CFLAGS=-g -I$(OPENSOAP_INCLUDE)
CGI_CFLAGS=$(CFLAGS) -DCONNECT_TYPE_CGI
CLIENT_LDFLAGS=-Wl,-rpath -Wl,$(OPENSOAP_LIB) -lOpenSOAPClient
SERVICE_LDFLAGS=$(CLIENT_LDFLAGS) -lOpenSOAPService

SOURCES = ProductList.c	ShoppingClMsgMgr.c ShoppingClMain.c ShoppingService.c
OBJS=$(SOURCES:.c=.o) ShoppingService.cgi.o


all: $(PROGRAMS)


## Make Client Programs

ShoppingClient: ProductList.o ShoppingClMsgMgr.o ShoppingClMain.o
	$(CC)  -o $@ $^ $(CLIENT_LDFLAGS)


## Make Service Programs

ShoppingService: ShoppingService.o
	$(CC)  -o $@ $^ $(SERVICE_LDFLAGS)

ShoppingService.cgi: ShoppingService.cgi.o
	$(CC)  -o $@ $^ $(SERVICE_LDFLAGS)

ShoppingService.cgi.o: ShoppingService.c
	$(CC) $(CGI_CFLAGS) -c -o $@ $<


## Install Service Programs

install: $(PROGRAMS)
	cp ShoppingService.cgi /home/httpd/cgi-bin/
	cp ShoppingService /usr/local/sbin/
	cp Shopping.ssml /var/tmp/OpenSOAP/conf/ssml/


clean: 
	rm -f $(OBJS) $(PROGRAMS)


## Test Service Programs for Success Cases

stest1:
	cat ShoppingSampleMessages/GetProductCountReq.xml | ./ShoppingService.cgi
stest2:	    
	cat ShoppingSampleMessages/GetProductSpecReq.xml | ./ShoppingService.cgi
stest3:	    
	cat ShoppingSampleMessages/GetStockQtyReq.xml | ./ShoppingService.cgi
stest4:	    
	cat ShoppingSampleMessages/PlaceOrderReq.xml | ./ShoppingService.cgi


## Test Service Programs for Fault Cases

stest5:
	cat ShoppingSampleMessages/GetProductCountReqF1.xml | ./ShoppingService.cgi
stest6:
	cat ShoppingSampleMessages/GetProductSpecReqF1.xml | ./ShoppingService.cgi
	cat ShoppingSampleMessages/GetProductSpecReqF2.xml | ./ShoppingService.cgi
	cat ShoppingSampleMessages/GetProductSpecReqF3.xml | ./ShoppingService.cgi
stest7:
	cat ShoppingSampleMessages/GetStockQtyReqF1.xml | ./ShoppingService.cgi
	cat ShoppingSampleMessages/GetStockQtyReqF2.xml | ./ShoppingService.cgi
	cat ShoppingSampleMessages/GetStockQtyReqF3.xml | ./ShoppingService.cgi
stest8:
	cat ShoppingSampleMessages/PlaceOrderReqF1.xml | ./ShoppingService.cgi
	cat ShoppingSampleMessages/PlaceOrderReqF2.xml | ./ShoppingService.cgi
	cat ShoppingSampleMessages/PlaceOrderReqF3.xml | ./ShoppingService.cgi
	cat ShoppingSampleMessages/PlaceOrderReqF4.xml | ./ShoppingService.cgi
	cat ShoppingSampleMessages/PlaceOrderReqF5.xml | ./ShoppingService.cgi


## Test Client-Service Hookups for Success Cases

ctest1:
	./ShoppingClient -s http://localhost/cgi-bin/ShoppingService.cgi -c EUC-JP GetProductList
ctest2:
	./ShoppingClient -s http://localhost/cgi-bin/ShoppingService.cgi -c EUC-JP GetStockQty NS01
ctest3:
	./ShoppingClient -s http://localhost/cgi-bin/ShoppingService.cgi -c EUC-JP PlaceOrder NS01 1


## Test Client-Server-Service Hookups for Success Cases

cstest1:
	./ShoppingClient -s http://localhost/cgi-bin/soapInterface.cgi -c EUC-JP GetProductList
cstest2:
	./ShoppingClient -s http://localhost/cgi-bin/soapInterface.cgi -c EUC-JP GetStockQty NS01
cstest3:
	./ShoppingClient -s http://localhost/cgi-bin/soapInterface.cgi -c EUC-JP PlaceOrder NS01 1
cstest4:
	./ShoppingClient -s http://localhost/cgi-bin/soapInterface.cgi -c EUC-JP PlaceOrderAsync NS01 1 > tmp_msg
cstest5:
	cat tmp_msg | ./ShoppingClient -s http://localhost/cgi-bin/soapInterface.cgi -c EUC-JP ConfirmOrder
