/*-----------------------------------------------------------------------------
 * $RCSfile: XMLNamespace.c,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#if (!defined(_WIN32)) || defined(_DEBUG)
static  const   char    CVS_ID[]        =       "$Id: XMLNamespace.c,v 1.24 2001/12/18 02:01:01 bandou Exp $";
#endif  /* _DEBUG */

#include "XMLNamespace.h"

/*
=begin
= OpenSOAP XML namespace class
=end
 */
int
OpenSOAPXMLNamespaceReleaseMembers(/* [in, out] */ OpenSOAPXMLNamespacePtr
								   ns) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (ns) {
        OpenSOAPStringRelease(ns->prefix);
        OpenSOAPStringRelease(ns->uri);

        ret = OPENSOAP_NO_ERROR;
    }

    return ret;
}

int
OpenSOAPXMLNamespaceFree(/* [in, out] */ OpenSOAPObjectPtr obj) {
    int ret = OpenSOAPXMLNamespaceReleaseMembers((OpenSOAPXMLNamespacePtr)obj);

    if (OPENSOAP_SUCCEEDED(ret)) {
        free(obj);
    }

    return ret;
}

int
OpenSOAPXMLNamespaceReleaseList(/* [in, out] */ OpenSOAPXMLNamespacePtr ns) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    while (ns) {
        OpenSOAPXMLNamespacePtr next = ns->next;
            
        OpenSOAPXMLNamespaceRelease(ns);

        ns = next;
        ret = OPENSOAP_NO_ERROR;
    }

    return ret;
}

int
OpenSOAPXMLNamespaceInitialize(/* [in, out] */ OpenSOAPXMLNamespacePtr ns) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (ns) {
        ns->prev = NULL;
        ns->next = NULL;

        ns->prefix = NULL;
        ns->uri    = NULL;
        ns->definedElement = NULL;

        ret = OpenSOAPObjectInitialize((OpenSOAPObjectPtr)ns,
                                       OpenSOAPXMLNamespaceFree);
        
    }

    return ret;
}


/*
=begin
--- function#OpenSOAPXMLNamespaceCreate(ns)
    OpenSOAP XML Namespace Element instance create(OpenSOAPString)

    :Parameters
      :[out] OpenSOAPXMLElmPtr * ((|ns|))
        XML Namespace.
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
 */
int
OpenSOAPXMLNamespaceCreate(/* [out] */ OpenSOAPXMLNamespacePtr *ns) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (ns) {
		ret = OPENSOAP_MEM_BADALLOC;
		*ns = malloc(sizeof(OpenSOAPXMLNamespace));
		if (*ns) {
			ret = OpenSOAPXMLNamespaceInitialize(*ns);
			if (OPENSOAP_FAILED(ret)) {
				free(*ns);
				*ns = NULL;
			}
		}
	}

	return ret;
}

static
int
OpenSOAPStringDuplicateAllowNULL(/* [in, out] */ OpenSOAPStringPtr *str) {
	int ret = OPENSOAP_PARAMETER_BADVALUE;

	if (str) {
		OpenSOAPStringPtr src = *str;
		*str = NULL;
		ret = (src) ? OpenSOAPStringDuplicate(src, str) : OPENSOAP_NO_ERROR;
	}

	return ret;
}

/*
=begin
--- function#OpenSOAPXMLNamespaceCreateStringWithFlag(ns_uri, ns_prefix, is_dup, ns)
    OpenSOAP XML Namespace Element instance create(OpenSOAPString)

    :Parameters
      :[in]  OpenSOAPStringPtr ((|ns_uri|))
        Namespace URI. this 
      :[in]  OpenSOAPStringPtr ((|ns_prefix|))
        Namespace prefix
      :[in]  int ((|is_dup|))
        parameter is duplicate flag.
      :[out] OpenSOAPXMLElmPtr * ((|ns|))
        XML Namespace.
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
 */
static
int
OpenSOAPXMLNamespaceCreateStringWithFlag(/* [in] */  OpenSOAPStringPtr ns_uri,
										 /* [in] */  OpenSOAPStringPtr ns_prefix,
										 /* [in] */  int is_dup,
										 /* [out] */ OpenSOAPXMLNamespacePtr *ns) {
    int ret = OpenSOAPXMLNamespaceCreate(ns);

	if (OPENSOAP_SUCCEEDED(ret)) {
		if (is_dup) {
			ret = OpenSOAPStringDuplicateAllowNULL(&ns_uri);
			if (OPENSOAP_SUCCEEDED(ret)) {
				ret = OpenSOAPStringDuplicateAllowNULL(&ns_prefix);
				if (OPENSOAP_FAILED(ret)) {
					OpenSOAPStringRelease(ns_uri);
				}
			}
		}
		if (OPENSOAP_SUCCEEDED(ret)) {
			(*ns)->uri    = ns_uri;
			(*ns)->prefix = ns_prefix;
		}
		else {
			OpenSOAPXMLNamespaceRelease(*ns);
			*ns = NULL;
		}
	}

	return ret;
}

/*
=begin
--- function#OpenSOAPXMLNamespaceCreateString(ns_uri, ns_prefix, is_dup, ns)
    OpenSOAP XML Namespace Element instance create(OpenSOAPString)

    :Parameters
      :[in]  OpenSOAPStringPtr ((|ns_uri|))
        Namespace URI. this 
      :[in]  OpenSOAPStringPtr ((|ns_prefix|))
        Namespace prefix
      :[out] OpenSOAPXMLElmPtr * ((|ns|))
        XML Namespace.
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
 */
extern
int
OPENSOAP_API
OpenSOAPXMLNamespaceCreateString(/* [in] */  OpenSOAPStringPtr ns_uri,
								 /* [in] */  OpenSOAPStringPtr ns_prefix,
								 /* [out] */ OpenSOAPXMLNamespacePtr *ns) {
	int ret = OpenSOAPXMLNamespaceCreateStringWithFlag(ns_uri,
													   ns_prefix,
													   1,
													   ns);

	return ret;
}

/*
=begin
--- function#OpenSOAPXMLNamespaceCreateMB(ns_uri, ns_prefix, ns)
    OpenSOAP XML Namespace Element instance create(MB)

    :Parameters
      :[in]  const char * ((|ns_uri|))
        Namespace URI
      :[in]  const char * ((|ns_prefix|))
        Namespace prefix
      :[out] OpenSOAPXMLElmPtr * ((|ns|))
        XML Namespace.
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
 */
extern
int
OPENSOAP_API
OpenSOAPXMLNamespaceCreateMB(/* [in] */  const char *ns_uri,
                             /* [in] */  const char *ns_prefix,
                             /* [out] */ OpenSOAPXMLNamespacePtr *ns) {
	OpenSOAPStringPtr ns_uri_str = NULL;
	int ret = ns_uri ? OpenSOAPStringCreateWithMB(ns_uri, &ns_uri_str)
		: OPENSOAP_NO_ERROR;

	if (OPENSOAP_SUCCEEDED(ret)) {
		OpenSOAPStringPtr ns_prefix_str = NULL;
		ret = ns_prefix ? OpenSOAPStringCreateWithMB(ns_prefix, &ns_prefix_str)
			: OPENSOAP_NO_ERROR;
		if (OPENSOAP_SUCCEEDED(ret)) {
			ret = OpenSOAPXMLNamespaceCreateStringWithFlag(ns_uri_str,
														   ns_prefix_str,
														   0,
														   ns);
			if (OPENSOAP_FAILED(ret)) {
				OpenSOAPStringRelease(ns_prefix_str);
			}
		}
		if (OPENSOAP_FAILED(ret)) {
			OpenSOAPStringRelease(ns_uri_str);
		}
	}

	return ret;
}

/*
=begin
--- function#OpenSOAPXMLNamespaceCreateWC(ns_uri, ns_prefix, ns)
    OpenSOAP XML Namespace Element instance create(WC)

    :Parameters
      :const wchar_t * [in] ((|ns_uri|))
        Namespace URI
      :const wchar_t * [in] ((|ns_prefix|))
        Namespace prefix
      :OpenSOAPXMLElmPtr * [out] ((|ns|))
        XML Namespace.
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
 */
extern
int
OPENSOAP_API
OpenSOAPXMLNamespaceCreateWC(/* [in] */  const wchar_t *ns_uri,
                             /* [in] */  const wchar_t *ns_prefix,
                             /* [out] */ OpenSOAPXMLNamespacePtr *ns) {
	OpenSOAPStringPtr ns_uri_str = NULL;
	int ret = ns_uri ? OpenSOAPStringCreateWithWC(ns_uri, &ns_uri_str)
		: OPENSOAP_NO_ERROR;

	if (OPENSOAP_SUCCEEDED(ret)) {
		OpenSOAPStringPtr ns_prefix_str = NULL;
		ret = ns_prefix ? OpenSOAPStringCreateWithWC(ns_prefix, &ns_prefix_str)
			: OPENSOAP_NO_ERROR;
		if (OPENSOAP_SUCCEEDED(ret)) {
			ret = OpenSOAPXMLNamespaceCreateStringWithFlag(ns_uri_str,
														   ns_prefix_str,
														   0,
														   ns);
			if (OPENSOAP_FAILED(ret)) {
				OpenSOAPStringRelease(ns_prefix_str);
			}
		}
		if (OPENSOAP_FAILED(ret)) {
			OpenSOAPStringRelease(ns_uri_str);
		}
	}

	return ret;
}

/*
=begin
--- function#OpenSOAPXMLNamespaceRelease(ns)
    Relese XML Namespace area

    :Parameters
      :[in, out] OpenSOAPXMLNamespacePtr ((|ns|))
        XML Namespace.
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
 */
extern
int
OPENSOAP_API
OpenSOAPXMLNamespaceRelease(/* [in, out] */ OpenSOAPXMLNamespacePtr ns) {
    int ret = OpenSOAPObjectRelease((OpenSOAPObjectPtr)ns);

    return ret;
}

/*
=begin
--- function#OpenSOAPXMLNamespaceGetURI(ns, ns_uri)
    Get Namespace URI

    :Parameters
      :[in]  OpenSOAPXMLNamespacePtr ((|ns|))
        XML Namespace.
      :[out] OpenSOAPStringPtr * ((|ns_uri|))
        Namespace URI
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
 */
extern
int
OPENSOAP_API
OpenSOAPXMLNamespaceGetURI(/* [in] */  OpenSOAPXMLNamespacePtr ns,
						   /* [out] */ OpenSOAPStringPtr *ns_uri) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if (!ns || !ns_uri) {
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	*ns_uri = ns->uri;

	ret = OPENSOAP_NO_ERROR;

    return ret;
}

/*
=begin
--- function#OpenSOAPXMLNamespaceGetPrefix(ns, ns_prefix)
    Get Namespace Prefix

    :Parameters
      :[in]  OpenSOAPXMLNamespacePtr ((|ns|))
        XML Namespace.
      :[out] OpenSOAPStringPtr * ((|ns_prefix|))
        Namespace Prefix
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
 */
extern
int
OPENSOAP_API
OpenSOAPXMLNamespaceGetPrefix(/* [in] */  OpenSOAPXMLNamespacePtr ns,
							  /* [out] */ OpenSOAPStringPtr *ns_prefix) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if (!ns || !ns_prefix) {
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	*ns_prefix = ns->prefix;

	ret = OPENSOAP_NO_ERROR;

    return ret;
}

