/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: DataRetainer.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#if HAVE_CONFIG_H
# include "config.h"
#endif

#include <time.h>
#include <cstdio>
#include <iostream>
#include <fstream>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#include <fcntl.h>

//#include <iterator>

#include "ServerCommon.h"
#include "DataRetainer.h"

using namespace OpenSOAP;

//for LOG
static const std::string CLASS_SIG = "IdManager";

const std::string DataRetainer::Id::INVALID = "<invalid>";

const DataRetainer::Id DataRetainer::getNewId() 
{
  typedef struct sockaddr SockAddr;
  typedef struct sockaddr_un SockAddrUn;

  int sockfd = int();
  SockAddrUn servAddr;

  //memset((char*)&servAddr, 0, sizeof(servAddr));
  servAddr.sun_family = AF_UNIX;
  strcpy(servAddr.sun_path, IDMGR_SOCKET_ADDR.c_str());
#ifdef DEBUG
  cerr << "sockAddr=[" << IDMGR_SOCKET_ADDR << "]" << endl;
#endif DEBUG
  //strcpy(servAddr.sun_path, "/var/tmp/OpenSOAP/SOCKET/IdManager");

  if (0 > (sockfd = socket(AF_UNIX, SOCK_STREAM, 0))) {
    cerr << TAG_ERR << CLASS_SIG 
	 << "::connectChannelManager open socket failed" << endl;
    return DataRetainer::Id::INVALID;
    //throw std::exception();
    // ONX̐݌vKv 
  }

#ifdef DEBUG
  cerr << CLASS_SIG << "::getNewId: socket open OK!" << endl;
#endif
  
#if HAVE_TYPE_OF_ADDRLEN_AS_SOCKLEN_T
  socklen_t servAddrLen = socklen_t();
#endif
#if HAVE_TYPE_OF_ADDRLEN_AS_SIZE_T
  size_t servAddrLen = size_t();
#endif
#if HAVE_TYPE_OF_ADDRLEN_AS_INT
  int servAddrLen = int();
#endif
  
  servAddrLen = sizeof(servAddr.sun_family) + strlen(servAddr.sun_path);
  
  if (0 > connect(sockfd, (SockAddr*)&servAddr, servAddrLen)) {
    cerr << TAG_ERR << CLASS_SIG 
	 << "::getNewId connect failed" << endl;
    return DataRetainer::Id::INVALID;
    //throw std::exception();
    // ONX̐݌vKv 
  }

#ifdef DEBUG
  cerr << CLASS_SIG << "::getNewId: connect OK!" << endl;
#endif

  std::string readBuf;
  if (0 > read(sockfd, readBuf)) {
    cerr << TAG_ERR << CLASS_SIG 
	 << "::getNewId read failed" << endl;
    return DataRetainer::Id::INVALID;
    //throw std::exception();
    // ONX̐݌vKv 
  }

#ifdef DEBUG
  cerr << CLASS_SIG << "::getNewId: read=[" << readBuf << "]" << endl;
#endif DEBUG
    
  return DataRetainer::Id(readBuf);
}
	
DataRetainer::DataRetainer(const std::map<int, std::string>& attr)
  : attributes_(attr)
  , saveDir_("/tmp")
{
  std::map<int, std::string>::const_iterator itr = attributes_.find(SAVE_DIR);
  if (itr != attributes_.end()) {
    saveDir_ = (*itr).second;
  }
}

DataRetainer::DataRetainer(const std::string& saveDir)
  : saveDir_(saveDir)
{
  attributes_.insert(make_pair(SAVE_DIR, saveDir));
}

DataRetainer::~DataRetainer()
{

}

int DataRetainer::setAttributes(const std::map<int, std::string>& attr)
{
  attributes_ = attr;
  std::map<int, std::string>::const_iterator itr = attributes_.find(SAVE_DIR);
  if (itr != attributes_.end()) {
    saveDir_ = (*itr).second;
  }
  return 0;
}


//f[^ۑ
int DataRetainer::put(DataContainer& dataCont)
{
  Id newId = getNewId();
  if (!newId.isValid()) {
    cerr << "invalid id" << endl;
    return -1;
  }
  dataCont.setId(newId);

  //o̓f[^`FbN
  std::string data = dataCont.getData();
  if (data.empty()) {
    cerr << "output data not set" << endl;
    return 0;
  }

  std::string fileId = saveDir_ + "/" + newId.toString();
  
  ofstream file(fileId.c_str());

  if (!file) {
    //t@CI[vs
    cerr << "can't open output file \""
	 << fileId << "\"" << endl;
    return -1;
  }

  //file << dataBuf_ << endl;
  file << data << flush;

  return data.length();
}

//f[^㏑
int DataRetainer::update(DataContainer& dataCont)
{
  Id id = dataCont.getId();
  if (!id.isValid() || !exists(id)) {
    return -1;
  }
  //o̓f[^`FbN
  std::string data = dataCont.getData();
  if (data.empty()) {
    cerr << "output data not set" << endl;
    return 0;
  }

  std::string fileId = saveDir_ + "/" + id.toString();
  
  ofstream file(fileId.c_str());

  if (!file) {
    //t@CI[vs
    cerr << "can't open output file \""
	 << fileId << "\"" << endl;
    return -1;
  }

  //file << dataBuf_ << endl;
  file << data << flush;

  return data.length();
  
}

//f[^ǂݏo
int DataRetainer::get(DataContainer& dataCont)
{
  Id id = dataCont.getId();
  if (!id.isValid() || !exists(id)) {
    return -1;
  }
  
  std::string fileId = saveDir_ + "/" + id.toString();

  std::ifstream file(fileId.c_str());

  if (!file) {
    //t@CI[vs
    cerr << "can't open input file \""
	 << fileId << "\"" << endl;
    return -1;
  }

  std::string data;
  char c;
  while (file.get(c)) {
    data += c;
  }
  
  dataCont.setData(data);

#ifdef DEBUG
  cerr << CLASS_SIG << "::get: READ=[" << data << "]" << endl;
#endif DEBUG

  return 0;
}

//f[^폜
int DataRetainer::del(const Id& id)
{
  if (!exists(id)) {
    return -1;
  }
  std::string fileId = saveDir_ + "/" + id.toString();
  return unlink(fileId.c_str());
}

//f[^ǂݏo폜
int DataRetainer::delget(DataContainer& dataCont)
{
  int ret = get(dataCont);
  if (0 > ret) {
    return ret;
  }
  return del(dataCont.getId());
}

//f[^݃`FbN
bool DataRetainer::exists(const Id& id)
{
  if (!id.isValid()) {
    return false;
  }
  
  std::string fileId = saveDir_ + "/" + id.toString();

  std::ifstream file(fileId.c_str());

  if (!file) {
    return false;
  }
  else {
    return true;
  }
}

// End of IPCDataCommunicator.cpp
