/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: MapUtil.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#include "MapUtil.h"
#include "ServerCommon.h"

using namespace OpenSOAP;

//for LOG
static const std::string CLASS_SIG = "MapUtil";

MapUtil::MapUtil(const std::string& itemDelimit, 
		 const std::string& keyValDelimit)
  : itemDelimit_(itemDelimit)
  , keyValDelimit_(keyValDelimit)
{
  
}

MapUtil::~MapUtil()
{

}

bool MapUtil::exists(const std::string& key) const
{
  std::map<std::string, std::string>::const_iterator itr = find(key);

  return itr == end() ? false : true;
}

std::string MapUtil::toString() const
{
  std::string retStr;
  std::map<std::string, std::string>::const_iterator mpos;
  for (mpos = begin(); mpos != end(); ++mpos) {
    if (!retStr.empty()) {
      retStr += itemDelimit_;
    }
    retStr += (*mpos).first;
    std::string val = (*mpos).second;
    if (!val.empty()) {
      retStr += keyValDelimit_;
    }
    retStr += val;
  }
  return retStr;
}

void MapUtil::insertItem(const std::string& data)
{
  std::string::size_type begIdx = 0;
  std::string::size_type endIdx = 0;
  for (;;) {
    begIdx = data.find_first_not_of(itemDelimit_, endIdx);
    if (begIdx == std::string::npos) {
#ifdef DEBUG
      cerr << "END" << endl;
#endif DEBUG
      break;
    }
    endIdx = data.find_first_of(itemDelimit_, begIdx);
    if (endIdx == std::string::npos) {
      endIdx = data.length();
    }
    std::string token = data.substr(begIdx, (endIdx-begIdx));

#ifdef DEBUG
    cerr << "token=[" << token << "]" << endl;
#endif DEBUG

    insertMap(token);
  }

}

void MapUtil::insertMap(const std::string& item)
#if 1
{
  std::string::size_type begIdx = 0;
  std::string::size_type endIdx = 0;

  begIdx = item.find_first_not_of(keyValDelimit_);
  if (begIdx == std::string::npos) {
    cerr << "no key" << endl;
    return;
  }
  endIdx = item.find_first_of(keyValDelimit_, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = item.length();
  }
  std::string key = item.substr(begIdx, (endIdx-begIdx));

#ifdef DEBUG
  cerr << "key=[" << key << "]" << endl;
#endif DEBUG

  std::string val;

  begIdx = item.find_first_not_of(keyValDelimit_, endIdx);
  if (begIdx == std::string::npos) {
    cerr << TAG_WRN << CLASS_SIG << "::insertMap: no value" << endl;
    //return;
    val = "";
  }
  else {
#if 1 // edit 2001/08/26
    // AAA=BBB=CCC  AAA : BBB=CCC Ə
    val = item.substr(begIdx, (item.length()-begIdx));
#else
    // AAA=BBB=CCC  AAA : BBB ƂȂĂ܂
    endIdx = item.find_first_of(keyValDelimit_, begIdx);
    if (endIdx == std::string::npos) {
      endIdx = item.length();
    }
    val = item.substr(begIdx, (endIdx-begIdx));
#endif
  }

#ifdef DEBUG
  cerr << "val=[" << val << "]" << endl;
#endif DEBUG
  
  insert(make_pair(key, val));

}
#else
{
  std::string::size_type begIdx = 0;
  std::string::size_type endIdx = 0;

  begIdx = item.find_first_not_of(keyValDelimit_);
  if (begIdx == std::string::npos) {
    cerr << "no key" << endl;
    return;
  }
  endIdx = item.find_first_of(keyValDelimit_, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = item.length();
  }
  std::string key = item.substr(begIdx, (endIdx-begIdx));

#ifdef DEBUG
  cerr << "key=[" << key << "]" << endl;
#endif DEBUG

  std::string val;

  begIdx = item.find_first_not_of(keyValDelimit_, endIdx);
  if (begIdx == std::string::npos) {
    cerr << TAG_WRN << CLASS_SIG << "::insertMap: no value" << endl;
    //return;
    val = "";
  }
  else {
    endIdx = item.find_first_of(keyValDelimit_, begIdx);
    if (endIdx == std::string::npos) {
      endIdx = item.length();
    }
    val = item.substr(begIdx, (endIdx-begIdx));
  }

#ifdef DEBUG
  cerr << "val=[" << val << "]" << endl;
#endif DEBUG
  
  insert(make_pair(key, val));

}
#endif //if 0

void MapUtil::spy() const
{
  cerr << "### MapUtil::spy() ###" << endl;
  cerr << " [KEY]:[VALUE]" << endl;
  cerr << " -------------" << endl;
  std::map<std::string, std::string>::const_iterator mpos;
  for (mpos = begin(); mpos != end(); ++mpos) {
    cerr << " [" << (*mpos).first << "]:[" 
	 << (*mpos).second << "]" << endl;
  }
  cerr << "######################" << endl;
}
