/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: SrvDrvChnlSelector.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#include <iostream>
#include <cstdio>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#include <fcntl.h>

#include "MapUtil.h"
#include "StringUtil.h"
#include "ServerCommon.h"
#include "ChannelCommon.h"
#include "SrvDrvChnlSelector.h"

using namespace OpenSOAP;

//for LOG
static const std::string CLASS_SIG = "SrvDrvChnlSelector";


SrvDrvChnlSelector::SrvDrvChnlSelector(bool asyncFlag)
{
#ifdef DEBUG
  cerr << CLASS_SIG << "::" << CLASS_SIG << endl;
#endif DEBUG  

  asyncFlag_ = asyncFlag;
  std::string sockAddr;
  
  if (asyncFlag_) {
    sockAddr = ASYNC_SRVDRV_SOCKET_ADDR;
  }
  else {
    sockAddr = SYNC_SRVDRV_SOCKET_ADDR;
  }
  //getMsgDrvChannelAddr();
  
  setSocketAddr(sockAddr);
}

SrvDrvChnlSelector::~SrvDrvChnlSelector()
{
#ifdef DEBUG
  cerr << CLASS_SIG << "::~" << CLASS_SIG << endl;
#endif DEBUG  

}

std::string SrvDrvChnlSelector::makeSendOpenMessage(const int waitSecond)
{
  MapUtil mapUtil(OpenSOAP::ChnlCommon::ITEM_DELMIT,
		  OpenSOAP::ChnlCommon::KEY_DELMIT);
  std::string s1(OpenSOAP::ChnlCommon::CMD);
  std::string s2(OpenSOAP::ChnlCommon::OPEN);
  mapUtil.insert(make_pair(s1,s2));
  if (ChannelDescriptor::NO_WAIT != waitSecond) {
    s1 = OpenSOAP::ChnlCommon::WAIT;
    s2 = StringUtil::toString(waitSecond);
    mapUtil.insert(make_pair(s1,s2));
  }
  return mapUtil.toString();
}

std::string SrvDrvChnlSelector::makeSendCloseMessage()
{
  MapUtil mapUtil(OpenSOAP::ChnlCommon::ITEM_DELMIT,
		  OpenSOAP::ChnlCommon::KEY_DELMIT);
  std::string s1(OpenSOAP::ChnlCommon::CMD);
  std::string s2(OpenSOAP::ChnlCommon::CLOSE);
  mapUtil.insert(make_pair(s1,s2));
  return mapUtil.toString();
}
  
// End of SrvDrvChnlSelector.cpp
