/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: XmlQuery.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#include <string>

#include <stdexcept>

#include <libxml/tree.h>
#include <libxml/parser.h>

#include "ServerCommon.h"
#include "SSMLAttrMgrProtocol.h"
#include "XmlQuery.h"

using namespace OpenSOAP;

//l̊i[ꏊ
//ꍇL
//std::vector<std::string> value;

//std::string nodeName_; //(K{)
//DataSet node_;

//̃m[h̗vflړIǂB
//vf͕K{ł邽߁AvfL肩lݒ(NULL)Ƃ
//ł͖ړIlǂfłȂ߁B
//bool isTarget_; 
    
//std::vector<DataSet> attributes_;

//XmlQuery* next_;

// for LOG
static const std::string CLASS_SIG = "XmlQuery";

//DOM
XmlQuery::XmlQuery(const std::string& query)
  : next_(NULL)
  , isTarget_(false)
  , isMulti_(false)
{
  std::string queryStr(query);

  //NODE_DELMIT(="/")ɂm[hKw̐؂o
  std::string::size_type begIdx = 0;
  std::string::size_type endIdx = 0;

  begIdx = queryStr.find_first_not_of(NODE_DELMIT);
  if (begIdx == std::string::npos) {
#ifdef DEBUG
    cerr << "END" << endl;
#endif DEBUG
    cerr << TAG_ERR << CLASS_SIG << "::" << CLASS_SIG
	 << ": invalid query data. query=[" << query << "]" << endl;
    throw std::exception();
  }
  
  endIdx = queryStr.find_first_of(NODE_DELMIT, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = queryStr.length();
  }

  //̃CX^X(Kw)ŏf[^
  std::string curQuery = queryStr.substr(begIdx, (endIdx-begIdx));

#ifdef DEBUG
  cerr << "curQuery=[" << curQuery << "]" << endl;
#endif DEBUG

  //̊Kwֈnf[^
  queryStr = queryStr.substr(endIdx);

#ifdef DEBUG
  cerr << "next queryStr=[" << queryStr << "]" << endl;
#endif DEBUG

  //-------------------------------------------------
  //vf(Kw), next_ɃCX^Xo^
  //-------------------------------------------------
  begIdx = queryStr.find_first_not_of(NODE_DELMIT);
  if (begIdx != std::string::npos) {
#ifdef DEBUG
    cerr << "next Query create." << endl;
#endif DEBUG

    next_ = new XmlQuery(queryStr);
  }

  //f[^
  //OƑ̐؂o
  begIdx = curQuery.find_first_not_of(ATTR_DELMIT);
  if (begIdx == std::string::npos) {
    cerr << TAG_ERR << CLASS_SIG << "::" << CLASS_SIG
	 << ": invalid current query data. query=[" << curQuery << "]" << endl;
    throw std::exception();
  }

  endIdx = curQuery.find_first_of(ATTR_DELMIT, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = curQuery.length();
  }
  //O
  std::string nameToken = curQuery.substr(begIdx, (endIdx-begIdx));
  addName(nameToken);

  for (;;) {
    begIdx = curQuery.find_first_not_of(ATTR_DELMIT, endIdx);
    if (begIdx == std::string::npos) {
#ifdef DEBUG
      cerr << "END" << endl;
#endif DEBUG
      break;
    }
    endIdx = curQuery.find_first_of(ATTR_DELMIT, begIdx);
    if (endIdx == std::string::npos) {
      endIdx = curQuery.length();
    }
    std::string attrToken = curQuery.substr(begIdx, (endIdx-begIdx));

#ifdef DEBUG
    cerr << "attrToken=[" << attrToken << "]" << endl;
#endif DEBUG

    addAttributes(attrToken);
  }


}

void XmlQuery::addName(const std::string& nameToken)
{
#ifdef DEBUG
  cerr << CLASS_SIG << "::addName() arg=[" << nameToken << "]" << endl;
#endif DEBUG

  //VAL_DELMIT(="=")ɂl̐؂o
  std::string::size_type begIdx = 0;
  std::string::size_type endIdx = 0;

  begIdx = nameToken.find_first_not_of(VAL_DELMIT);
  if (begIdx == std::string::npos) {
    cerr << TAG_ERR << CLASS_SIG << "::" << CLASS_SIG
	 << ": invalid name token data. nameToken=[" << nameToken << "]" << endl;
    throw std::exception();
  }
  
  endIdx = nameToken.find_first_of(VAL_DELMIT, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = nameToken.length();
  }
  
  //O
  std::string name = nameToken.substr(begIdx, (endIdx-begIdx));
  //node_.setName(nameToken.substr(begIdx, (endIdx-begIdx)));

  //̐؂o
  begIdx = nameToken.find_first_not_of(VAL_DELMIT, endIdx);
  if (begIdx == std::string::npos) {
    //w薳
#ifdef DEBUG
    cerr << CLASS_SIG << "::addName: no name condition." << endl;
#endif DEBUG

    //added proc. for get child tag names #2001/09/10
    if (QUESTION_MARK == name) {
      //Ώ
      isTarget_ = true;
      node_.setName(name);
    } 
    else if (MULTI_QUESTION_MARK == name) {
      //Ώ()
      isTarget_ = true;
      isMulti_ = true;
      //Ȍ̊mFɂMULTI_QUESTION_MARKgȂ
      node_.setName(QUESTION_MARK);
    }
    else {
      node_.setName(name);
    }
    return;
  }
  //O̒ʏݒ
  node_.setName(name);

  endIdx = nameToken.find_first_of(VAL_DELMIT, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = nameToken.length();
  }
  std::string cond = nameToken.substr(begIdx, (endIdx-begIdx));

#ifdef DEBUG
  cerr << CLASS_SIG << "::addName: name cond=[" << cond << "]" << endl;
#endif DEBUG

  if (QUESTION_MARK == cond) {
    //Ώ
    isTarget_ = true;
    node_.setValue(cond);
  } 
  else if (MULTI_QUESTION_MARK == cond) {
    //Ώ()
    isTarget_ = true;
    isMulti_ = true;
    //Ȍ̊mFɂMULTI_QUESTION_MARKgȂ
    node_.setValue(QUESTION_MARK);
  }
  else {
    node_.setValue(cond);
  }
}

void XmlQuery::addAttributes(const std::string& attrToken)
{
#ifdef DEBUG
  cerr << CLASS_SIG << "::addAttributes() arg=[" << attrToken << "]" << endl;
#endif DEBUG

  //VAL_DELMIT(="=")ɂl̐؂o
  std::string::size_type begIdx = 0;
  std::string::size_type endIdx = 0;
  
  begIdx = attrToken.find_first_not_of(VAL_DELMIT);
  if (begIdx == std::string::npos) {
    cerr << TAG_ERR << CLASS_SIG << "::" << CLASS_SIG
	 << ": invalid attr token data. attrToken=[" << attrToken << "]" << endl;
    throw std::exception();
  }
  
  endIdx = attrToken.find_first_of(VAL_DELMIT, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = attrToken.length();
  }
  
  DataSet attrDataSet;

  //
  std::string attrName = attrToken.substr(begIdx, (endIdx-begIdx));
  attrDataSet.setName(attrName);

  //̐؂o
  begIdx = attrToken.find_first_not_of(VAL_DELMIT, endIdx);
  if (begIdx == std::string::npos) {
    //w薳
    cerr << TAG_ERR << CLASS_SIG 
	 << "::addAttributes: no attr condition. attr=[" 
	 << attrName << "]" << endl;
    throw std::exception();
  }

  endIdx = attrToken.find_first_of(VAL_DELMIT, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = attrToken.length();
  }
  std::string cond = attrToken.substr(begIdx, (endIdx-begIdx));

#ifdef DEBUG
  cerr << CLASS_SIG << "::addAttributes: attr cond=[" << cond << "]" << endl;
#endif DEBUG

  if (QUESTION_MARK == cond) {
    //Ώ
    isTarget_ = true;
    attrDataSet.setValue(cond);
  } 
  else if (MULTI_QUESTION_MARK == cond) {
    //Ώ()
    isTarget_ = true;
    isMulti_ = true;
    //Ȍ̊mFɂMULTI_QUESTION_MARKgȂ
    attrDataSet.setValue(QUESTION_MARK);
  }
  else {
    attrDataSet.setValue(cond);
  }

  //ǉ
  attributes_.push_back(attrDataSet);
}

XmlQuery::~XmlQuery()
{
  if (next_) { delete next_; }
}
    
//l̎擾
//擾łtrueԂ
//RootQuery  RootNoden
//int XmlQuery::getValue(xmlDocPtr& doc, xmlNodePtr node)
int XmlQuery::getValue(xmlDocPtr& doc, xmlNodePtr node,
		       std::vector<std::string>& values)
{
  //bool result = false;
  int result = -101; // -101 = target not found

  //̈ꎞێobt@
  std::string answer;
  //擾σtO
  //bool hasAnswer = false;

  int i = 0;

  while (NULL != node) {
    
    //check node name
    if (QUESTION_MARK == node_.getName()) {
      //request tag name 
      answer = (const char*)node->name;
    }
    else {
      if (0 != strcmp((const char*)node->name, node_.getName().c_str())) {
	//ΏۊOm[h
	node = node->next;
	continue;
      }
    
      //check node element
      if (!node_.isNullValue()) {
	
	//vfwL̏ꍇ
	xmlChar* element = xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
	//      if (NULL == element) {
	//	return -102; // -102 = wvf
	//      }
	if (QUESTION_MARK == node_.getValue()) {
	  //ړI̓Ȃ΃obt@ɐݒ
	  if (NULL == element) {
	    answer = "";
	  }
	  else {
	    answer = (const char*)element;
	  }
	  //hasAnswer = true;
	}
	else if (0 != strcmp((const char*)element, node_.getValue().c_str())) {
	  //`FbN
	  //ΏۊOm[h
	  node = node->next;
	  continue;
	}
      }
     
      //check attributes
      std::vector<DataSet>::const_iterator pos;
      bool isValidAttr = true;
      for (pos = attributes_.begin(); pos != attributes_.end(); ++pos) {
	
	xmlChar* attr = xmlGetProp(node, (const xmlChar*)pos->getName().c_str());
	if (NULL == attr) {
	return -103; // -103 = w葮
	}
	if (QUESTION_MARK == pos->getValue()) {
	  //ړI̓Ȃ΃obt@ɐݒ
	  /*
	    if (hasAnswer) {
	    cerr << TAG_WRN << CLASS_SIG 
	    << "::getValue: answer alread set." << endl;
	    }
	  */
	  answer = (const char*)attr;
	  //hasAnswer = true;
	}
	else if (0 != strcmp((const char*)attr, pos->getValue().c_str())) {
	  //`FbN
	  //ΏۊOm[h
	  isValidAttr = false;
	  break;
	}
      } // end for ();
      if (!isValidAttr) {
	node = node->next;
	continue;
      }
      //܂ł΂̃m[h͏Ώ
      //qCX^Xֈn
      if (next_) {
	
	if (isTarget_) {
	  cerr << TAG_WRN << CLASS_SIG << 
	    "::getValue: current instance is target, but next_ has..." << endl;
	}
	return next_->getValue(doc, node->xmlChildrenNode, values);
      }
    }
    
    //o^
    values.push_back(answer);
    result = 0; // 0 = SUCCESS

    //񓚂łȂΏI
    if (!isMulti_) {
      break;
    }

    //̃m[h
    node = node->next;
    
    i++;
  }

  cerr << "return =(" << result << ")" << endl;
  return result;
}

void XmlQuery::spy() const
{
  cerr << CLASS_SIG << "::spy() #####" << endl;
  cerr << CLASS_SIG << "::node_:" << endl;
  node_.spy();

  cerr << "  isTarget_:" << isTarget_ << endl;
  cerr << "  isMulti_:" << isMulti_ << endl;

  cerr << CLASS_SIG << "::attributes_:" << endl;
  std::vector<DataSet>::const_iterator pos;
  for (pos = attributes_.begin(); pos != attributes_.end(); ++pos) {
    cerr << " -------------" << endl;   
    pos->spy();
  }
  
  if (next_) { next_->spy(); }
}

bool XmlQuery::isMulti() const
{
  if (isTarget_) {
    return isMulti_;
  }
  for (XmlQuery* cur = next_; NULL != cur; cur = cur->next_) {
    if (cur->isTarget_) {
      return cur->isMulti_;
    }
  }
  //warning
  cerr << TAG_WRN << CLASS_SIG << "::isMulti: target not found." << endl;

  return false;
}

